@@ -1,5 +1,6 @@
 /* [argparse.c wk 17.06.97] Argument Parser for option handling
- * Copyright (C) 1998, 1999, 2000, 2001, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2006
+ *               2007  Free Software Foundation, Inc.
  *
  * This file is part of JNLIB.
  *
@@ -29,6 +30,9 @@
 #include "mischelp.h"
 #include "stringhelp.h"
 #include "logging.h"
+#ifdef JNLIB_NEED_UTF8CONV
+#include "utf8conv.h"
+#endif
 #include "argparse.h"
 
 
@@ -212,16 +216,18 @@ initialize( ARGPARSE_ARGS *arg, const char *filename, unsigned *lineno )
 static void
 store_alias( ARGPARSE_ARGS *arg, char *name, char *value )
 {
-    /* TODO: replace this dummy function with a rea one
-     * and fix the probelms IRIX has with (ALIAS_DEV)arg..
-     * used as lvalue
-     */
+  (void)arg;
+  (void)name;
+  (void)value;
+  /* TODO: replace this dummy function with a real one
+   * and fix the problems IRIX has with (ALIAS_DEV)arg..
+   * used as lvalue.  */
 #if 0
-    ALIAS_DEF a = jnlib_xmalloc( sizeof *a );
-    a->name = name;
-    a->value = value;
-    a->next = (ALIAS_DEF)arg->internal.aliases;
-    (ALIAS_DEF)arg->internal.aliases = a;
+  ALIAS_DEF a = jnlib_xmalloc( sizeof *a );
+  a->name = name;
+  a->value = value;
+  a->next = (ALIAS_DEF)arg->internal.aliases;
+  (ALIAS_DEF)arg->internal.aliases = a;
 #endif
 }
 
@@ -429,6 +435,8 @@ find_long_option( ARGPARSE_ARGS *arg,
     int i;
     size_t n;
 
+    (void)arg;
+
     /* Would be better if we can do a binary search, but it is not
        possible to reorder our option table because we would mess
        up our help strings - What we can do is: Build a nice option
@@ -438,7 +446,7 @@ find_long_option( ARGPARSE_ARGS *arg,
     for(i=0; opts[i].short_opt; i++ )
 	if( opts[i].long_opt && !strcmp( opts[i].long_opt, keyword) )
 	    return i;
-  #if 0
+#if 0
     {
 	ALIAS_DEF a;
 	/* see whether it is an alias */
@@ -450,7 +458,7 @@ find_long_option( ARGPARSE_ARGS *arg,
 	    }
 	}
     }
-  #endif
+#endif
     /* not found, see whether it is an abbreviation */
     /* aliases may not be abbreviated */
     n = strlen( keyword );
@@ -699,18 +707,28 @@ set_opt_arg(ARGPARSE_ARGS *arg, unsigned flags, char *s)
 static size_t
 long_opt_strlen( ARGPARSE_OPTS *o )
 {
-    size_t n = strlen(o->long_opt);
+  size_t n = strlen (o->long_opt);
 
-    if( o->description && *o->description == '|' ) {
-	const char *s;
-
-	s=o->description+1;
-	if( *s != '=' )
-	    n++;
-	for(; *s && *s != '|'; s++ )
-	    n++;
+  if ( o->description && *o->description == '|' ) 
+    {
+      const char *s;
+#ifdef JNLIB_NEED_UTF8CONV
+      int is_utf8 = is_native_utf8 ();
+#endif
+        
+      s=o->description+1;
+      if ( *s != '=' )
+        n++;
+      /* For a (mostly) correct length calculation we exclude
+         continuation bytes (10xxxxxx) if we are on a native utf8
+         terminal. */
+      for (; *s && *s != '|'; s++ )
+#ifdef JNLIB_NEED_UTF8CONV
+        if ( is_utf8 && (*s&0xc0) != 0x80 )
+#endif
+          n++;
     }
-    return n;
+  return n;
 }
 
 /****************
@@ -820,20 +838,21 @@ show_help( ARGPARSE_OPTS *opts, unsigned flags )
 	    puts("\n(A single dash may be used instead of the double ones)");
     }
     if( (s=strusage(19)) ) {  /* bug reports to ... */
-        /* To allow changing the bug reporting address without
-         * touching an i18n string, we replace a placeholder @EMAIL@
-         * while printing by a string given with strusage(49). */
-        const char *email, *point;
+        char *s2;
+
 	putchar('\n');
-        email = strusage (49);
-        if (email && *email && (point = strstr (s, "@EMAIL@")) ) {
-            for (; s < point; s++)
-                putchar (*s);
-            fputs (email, stdout);
-            s = point + 7;
-        }
-        fputs(s, stdout);
-        
+        s2 = strstr (s, "@EMAIL@");
+        if (s2)
+          {
+            if (s2-s)
+              fwrite (s, s2-s, 1, stdout);
+            fputs (PACKAGE_BUGREPORT, stdout);
+            s2 += 7;
+            if (*s2)
+              fputs (s2, stdout);
+          }
+        else
+          fputs(s, stdout);
     }
     fflush(stdout);
     exit(0);
@@ -900,13 +919,11 @@ usage( int level )
  *    16: Long copying conditions (with LFs)
  *    17: Optional printable OS name
  *    18: Optional thanks list	 (with LFs)
- *    19: Bug report info (see also 190).
+ *    19: Bug report info
  *20..29: Additional lib version strings.
  *30..39: Additional program info (with LFs)
  *    40: short usage note (with LF)
  *    41: long usage note (with LF)
- *    49: bug reporting address: If not empty, insert this string 
- *        into the string returned by 19 at @EMAIL@
  */
 const char *
 strusage( int level )
@@ -919,7 +936,7 @@ strusage( int level )
     switch( level ) {
       case 11: p = "foo"; break;
       case 13: p = "0.0"; break;
-      case 14: p = "Copyright (C) 2006 Free Software Foundation, Inc."; break;
+      case 14: p = "Copyright (C) 2007 Free Software Foundation, Inc."; break;
       case 15: p =
 "This program comes with ABSOLUTELY NO WARRANTY.\n"
 "This is free software, and you are welcome to redistribute it\n"
@@ -968,17 +985,20 @@ main(int argc, char **argv)
 {
     ARGPARSE_OPTS opts[] = {
     { 'v', "verbose",   0 , "Laut sein"},
-    { 'e', "echo"   ,   0 , "Zeile ausgeben, damit wir sehen, was wir einegegeben haben"},
-    { 'd', "debug",     0 , "Debug\nfalls mal etasws\nSchief geht"},
+    { 'e', "echo"   ,   0 , ("Zeile ausgeben, damit wir sehen, was wir ein"
+                             " gegeben haben")},
+    { 'd', "debug",     0 , "Debug\nfalls mal etwas\nschief geht"},
     { 'o', "output",    2   },
     { 'c', "cross-ref", 2|8, "cross-reference erzeugen\n" },
+    /* Note that on a non-utf8 terminal the ß might garble the output. */
+    { 's', "street",  0,     "|Straße|set the name of the street to Straße" },
     { 'm', "my-option", 1|8 },
     { 500, "a-long-option", 0 },
     {0} };
     ARGPARSE_ARGS pargs = { &argc, &argv, 2|4|32 };
     int i;
 
-    while( ArgParse( &pargs, opts) ) {
+    while( arg_parse ( &pargs, opts) ) {
 	switch( pargs.r_opt ) {
 	  case -1 : printf( "arg=`%s'\n", pargs.r.ret_str); break;
 	  case 'v': opt.verbose++; break;
