@@ -1,5 +1,5 @@
 /* certcache.c - Certificate caching
- *      Copyright (C) 2004, 2005 g10 Code GmbH
+ *      Copyright (C) 2004, 2005, 2007 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -66,9 +66,10 @@ static cert_item_t cert_cache[256];
 
 /* This is the global cache_lock variable. In general looking is not
    needed but it would take extra efforts to make sure that no
-   indirect use of pth functions is done, so we simply lock it
-   always.  */
-static pth_rwlock_t cert_cache_lock = PTH_RWLOCK_INIT;
+   indirect use of pth functions is done, so we simply lock it always.
+   Note: We can't use static initialization, as that is not available
+   through w32-pth.  */
+static pth_rwlock_t cert_cache_lock;
 
 /* Flag to track whether the cache has been initialized.  */
 static int initialization_done;
@@ -80,7 +81,15 @@ static unsigned int total_extra_certificates;
 
 
 
-/* Helper to do the cahce locking.  */
+/* Helper to do the cache locking.  */
+static void
+init_cache_lock (void)
+{
+  if (!pth_rwlock_init (&cert_cache_lock))
+    log_fatal (_("can't initialize certificate cache lock: %s\n"),
+	       strerror (errno));
+}
+
 static void
 acquire_cache_read_lock (void)
 {
@@ -404,6 +413,7 @@ cert_cache_init (void)
   
   if (initialization_done)
     return;
+  init_cache_lock ();
   acquire_cache_write_lock ();
 
   dname = make_filename (opt.homedir, "trusted-certs", NULL);
