@@ -1,5 +1,5 @@
 /* dirmngr-client.c  -  A client for the dirmngr daemon
- *	Copyright (C) 2004 g10 Code GmbH
+ *	Copyright (C) 2004, 2007 g10 Code GmbH
  *	Copyright (C) 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of DirMngr.
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -59,6 +60,7 @@ enum
     oSquidMode,
     oPEM,
     oEscapedPEM,
+    oForceDefaultResponder
   };
 
 
@@ -74,10 +76,11 @@ static ARGPARSE_OPTS opts[] = {
   { oLoadCRL,  "load-crl",  0, N_("load a CRL into the dirmngr")},
   { oSquidMode,"squid-mode",0, N_("special mode for use by Squid")},
   { oPEM,      "pem",       0, N_("certificates are expected in PEM format")},
+  { oForceDefaultResponder, "force-default-responder", 0,
+    N_("force the use of the default OCSP responder")},
   { 0 }
 };
  
-#define DEFAULT_SOCKET_NAME "/var/run/dirmngr/socket"
 
 /* The usual structure for the program flags.  */
 static struct
@@ -86,11 +89,12 @@ static struct
   int verbose;
   const char *dirmngr_program;
   int force_pipe_server;
+  int force_default_responder;
   int pem;
   int escaped_pem; /* PEM is additional percent encoded.  */
 
   int use_ocsp;
-
+  
 } opt;
 
 
@@ -192,6 +196,16 @@ main (int argc, char **argv )
   set_strusage (my_strusage);
   log_set_prefix ("dirmngr-client",
                   JNLIB_LOG_WITH_PREFIX); 
+
+  /* For W32 we need to initialize the socket subsystem.  Becuase we
+     don't use Pth we need to do this explicit. */
+#ifdef HAVE_W32_SYSTEM  
+ {
+   WSADATA wsadat;
+
+   WSAStartup (0x202, &wsadat);
+ }
+#endif /*HAVE_W32_SYSTEM*/
   
   /* Init Assuan.  */
   assuan_set_assuan_log_stream (log_get_stream ());
@@ -224,6 +238,7 @@ main (int argc, char **argv )
           opt.escaped_pem = 1;
           cmd_squid_mode = 1;
           break;
+        case oForceDefaultResponder: opt.force_default_responder = 1; break;
 
         default : pargs.err = 2; break;
 	}
@@ -402,6 +417,8 @@ main (int argc, char **argv )
 static int
 status_cb (void *opaque, const char *line)
 {
+  (void)opaque;
+
   if (opt.verbose > 2)
     log_info (_("got status: `%s'\n"), line);
   return 0;
@@ -438,7 +455,7 @@ start_dirmngr (int only_daemon)
   infostr = opt.force_pipe_server? NULL : getenv ("DIRMNGR_INFO");
   if (only_daemon && (!infostr || !*infostr))
     {
-      infostr = DEFAULT_SOCKET_NAME;
+      infostr = xstrdup (default_socket_name ());
       try_default = 1;
     }
   if (!infostr || !*infostr)
@@ -801,9 +818,11 @@ do_check (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
   parm.cert = cert;
   parm.certlen = certlen;
 
-  err = assuan_transact (ctx, opt.use_ocsp? "CHECKOCSP":"CHECKCRL", NULL, NULL,
-                        inq_cert, &parm,
-                        status_cb, NULL);
+  err = assuan_transact (ctx, 
+                         (opt.use_ocsp && opt.force_default_responder
+                          ? "CHECKOCSP --force-default-responder" 
+                          : opt.use_ocsp? "CHECKOCSP" : "CHECKCRL"), 
+                         NULL, NULL, inq_cert, &parm, status_cb, NULL);
   if (opt.verbose > 1)
     log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
   return err;
