@@ -30,8 +30,14 @@
 #include <assert.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <ldap.h>
 
+#ifdef HAVE_W32_SYSTEM
+#include <winsock2.h>
+#include <winldap.h>
+#include "ldap-url.h"
+#else
+#include <ldap.h>
+#endif
 
 
 #define JNLIB_NEED_LOG_LOGV
@@ -42,8 +48,8 @@
 #include "../jnlib/strlist.h"
 
 #include "i18n.h"
-#include "util.h"
 #include "no-libgcrypt.h"
+#include "util.h"
 
 #define DEFAULT_LDAP_TIMEOUT 100 /* Arbitrary long timeout. */
 
@@ -255,17 +261,19 @@ main (int argc, char **argv )
 
   if (opt.alarm_timeout)
     {
-#if defined(HAVE_SIGACTION) && defined(HAVE_STRUCT_SIGACTION)
+#ifndef HAVE_W32_SYSTEM
+# if defined(HAVE_SIGACTION) && defined(HAVE_STRUCT_SIGACTION)
       struct sigaction act;
       
       act.sa_handler = catch_alarm;
       sigemptyset (&act.sa_mask);
       act.sa_flags = 0;
       if (sigaction (SIGALRM,&act,NULL))
-#else 
+# else 
       if (signal (SIGALRM, catch_alarm) == SIG_ERR)
-#endif
+# endif
           log_fatal ("unable to register timeout handler\n");
+#endif
     }
 
   for (; argc; argc--, argv++)
@@ -279,6 +287,7 @@ main (int argc, char **argv )
 static void
 catch_alarm (int dummy)
 {
+  (void)dummy;
   _exit (10);
 }
 
@@ -286,12 +295,14 @@ catch_alarm (int dummy)
 static void
 set_timeout (void)
 {
+#ifndef HAVE_W32_SYSTEM
+  /* FIXME for W32.  */
   if (opt.alarm_timeout)
     alarm (opt.alarm_timeout);
+#endif
 }
 
 
-
 /* Helper for fetch_ldap().  */
 static int
 print_ldap_entries (LDAP *ld, LDAPMessage *msg, char *want_attr)
@@ -333,22 +344,31 @@ print_ldap_entries (LDAP *ld, LDAPMessage *msg, char *want_attr)
 
           /* I case we want only one attribute we do a case
              insensitive compare without the optional extension
-             (i.e. ";binary").  case insensive is not really correct
-             but the best we can do. */
+             (i.e. ";binary").  Case insensitive is not really correct
+             but the best we can do.  */
           if (want_attr)
             {
-              char *cp = strchr (attr, ';');
-              if (cp)
-                *cp = 0;
-              if ( ascii_strcasecmp (want_attr, attr) )
+              char *cp1, *cp2;
+              int cmpres;
+
+              cp1 = strchr (want_attr, ';');
+              if (cp1)
+                *cp1 = 0;
+              cp2 = strchr (attr, ';');
+              if (cp2)
+                *cp2 = 0;
+              cmpres = ascii_strcasecmp (want_attr, attr);
+              if (cp1)
+                *cp1 = ';';
+              if (cp2)
+                *cp2 = ';';
+              if (cmpres)
                 {
                   ldap_memfree (attr);
                   continue; /* Not found:  Try next attribute.  */
                 }
-              if (cp)
-                *cp = ';';
             }
-          
+
           values = ldap_get_values_len (ld, item, attr);
   
           if (!values)
@@ -549,7 +569,7 @@ fetch_ldap (const char *url, const LDAPURLDesc *ludp)
 
 
 
-/* Main processing.  Take the URL and run the DLAP query. The result
+/* Main processing.  Take the URL and run the LDAP query. The result
    is printed to stdout, errors are logged to the log stream. */
 static int
 process_url (const char *url)
