@@ -1,5 +1,5 @@
 /* ocsp.c - OCSP management
- *      Copyright (C) 2004 g10 Code GmbH
+ *      Copyright (C) 2004, 2007 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -29,8 +29,8 @@
 #include "http.h"
 #include "validate.h"
 #include "certcache.h"
-
 #include "ocsp.h"
+#include "estream.h"
 
 /* The maximum size we allow as a response from an OCSP reponder. */
 #define MAX_RESPONSE_SIZE 65536
@@ -53,7 +53,7 @@ static const char oidstr_certHash[] = "1.3.36.8.3.13";
 /* Read from FP and return a newly allocated buffer in R_BUFFER with the
    entire data read from FP. */
 static gpg_error_t
-read_response (FILE *fp, unsigned char **r_buffer, size_t *r_buflen)
+read_response (estream_t fp, unsigned char **r_buffer, size_t *r_buflen)
 {
   gpg_error_t err;
   unsigned char *buffer;
@@ -74,8 +74,8 @@ read_response (FILE *fp, unsigned char **r_buffer, size_t *r_buflen)
       size_t nread = 0;
 
       assert (nbytes < bufsize);
-      nread = fread (buffer+nbytes, 1, bufsize-nbytes, fp);
-      if (nread < bufsize-nbytes && ferror (fp))
+      nread = es_fread (buffer+nbytes, 1, bufsize-nbytes, fp);
+      if (nread < bufsize-nbytes && es_ferror (fp))
         {
           err = gpg_error_from_errno (errno);
           log_error (_("error reading from responder: %s\n"),
@@ -83,7 +83,7 @@ read_response (FILE *fp, unsigned char **r_buffer, size_t *r_buflen)
           xfree (buffer);
           return err;
         }
-      if ( !(nread == bufsize-nbytes && !feof (fp)))
+      if ( !(nread == bufsize-nbytes && !es_feof (fp)))
         { /* Response succesfully received. */
           nbytes += nread;
           *r_buffer = buffer;
@@ -131,6 +131,8 @@ do_ocsp_request (ctrl_t ctrl, ksba_ocsp_t ocsp, gcry_md_hd_t md,
   int redirects_left = 2;
   char *free_this = NULL;
 
+  (void)ctrl;
+
   if (opt.disable_http)
     {
       log_error (_("OCSP request not possible due to disabled HTTP\n"));
@@ -175,12 +177,12 @@ do_ocsp_request (ctrl_t ctrl, ksba_ocsp_t ocsp, gcry_md_hd_t md,
       return err;
     }
 
-  fprintf (http_get_write_ptr (http),
-           "Content-Type: application/ocsp-request\r\n"
-           "Content-Length: %lu\r\n", 
-           (unsigned long)requestlen );
+  es_fprintf (http_get_write_ptr (http),
+	      "Content-Type: application/ocsp-request\r\n"
+	      "Content-Length: %lu\r\n", 
+	      (unsigned long)requestlen );
   http_start_data (http);
-  if (fwrite (request, requestlen, 1, http_get_write_ptr (http)) != 1)
+  if (es_fwrite (request, requestlen, 1, http_get_write_ptr (http)) != 1)
     {
       err = gpg_error_from_errno (errno);
       log_error ("error sending request to `%s': %s\n", url, strerror (errno));
@@ -298,26 +300,29 @@ do_ocsp_request (ctrl_t ctrl, ksba_ocsp_t ocsp, gcry_md_hd_t md,
 
 
 /* Validate that CERT is indeed valid to sign an OCSP response. If
-   signer_fpr is not NULL we simply check that CERT matches this
-   fingerprint. */
+   SIGNER_FPR_LIST is not NULL we simply check that CERT matches one
+   of the fingerprints in this list. */
 static gpg_error_t
-validate_responder_cert (ctrl_t ctrl, ksba_cert_t cert, const char *signer_fpr)
+validate_responder_cert (ctrl_t ctrl, ksba_cert_t cert, 
+                         fingerprint_list_t signer_fpr_list)
 {
   gpg_error_t err;
   char *fpr;
 
-  if (signer_fpr)
+  if (signer_fpr_list)
     {
-      fpr = (strchr (signer_fpr, ':')
-             ? get_fingerprint_hexstring_colon (cert)
-             : get_fingerprint_hexstring (cert));
-      if (ascii_strcasecmp (signer_fpr, fpr))
+      fpr = get_fingerprint_hexstring (cert);
+      for (; signer_fpr_list && strcmp (signer_fpr_list->hexfpr, fpr); 
+           signer_fpr_list = signer_fpr_list->next)
+        ;
+      if (signer_fpr_list)
+        err = 0;
+      else
         {
-          log_error (_("not signed by default OCSP signer certificate"));
+          log_error (_("not signed by a default OCSP signer's certificate"));
           err = gpg_error (GPG_ERR_BAD_CA_CERT);
         }
-      else
-        err = 0;
+      xfree (fpr);
     }
   else if (opt.system_daemon)
     {
@@ -354,7 +359,7 @@ validate_responder_cert (ctrl_t ctrl, ksba_cert_t cert, const char *signer_fpr)
 /* Helper for check_signature. */
 static int
 check_signature_core (ctrl_t ctrl, ksba_cert_t cert, gcry_sexp_t s_sig,
-                      gcry_sexp_t s_hash, const char *signer_fpr)
+                      gcry_sexp_t s_hash, fingerprint_list_t signer_fpr_list)
 {
   gpg_error_t err;
   ksba_sexp_t pubkey;
@@ -369,7 +374,7 @@ check_signature_core (ctrl_t ctrl, ksba_cert_t cert, gcry_sexp_t s_sig,
   if (!err)
     err = gcry_pk_verify (s_sig, s_hash, s_pkey);
   if (!err)
-    err = validate_responder_cert (ctrl, cert, signer_fpr);
+    err = validate_responder_cert (ctrl, cert, signer_fpr_list);
   if (!err)
     {
       gcry_sexp_release (s_pkey);
@@ -385,13 +390,13 @@ check_signature_core (ctrl_t ctrl, ksba_cert_t cert, gcry_sexp_t s_sig,
 /* Check the signature of an OCSP repsonse.  OCSP is the context,
    S_SIG the signature value and MD the handle of the hash we used for
    the response.  This function automagically finds the correct public
-   key.  If SIGNER_FPR is not NULL, the default OCSP reponder has been
-   used and thus the certificate to be used is the one identified by
-   this fingerprint. */
+   key.  If SIGNER_FPR_LIST is not NULL, the default OCSP reponder has been
+   used and thus the certificate is one of those identified by
+   the fingerprints. */
 static gpg_error_t
 check_signature (ctrl_t ctrl,
                  ksba_ocsp_t ocsp, gcry_sexp_t s_sig, gcry_md_hd_t md,
-                 const char *signer_fpr)
+                 fingerprint_list_t signer_fpr_list)
 {
   gpg_error_t err;
   int algo, cert_idx;
@@ -418,16 +423,18 @@ check_signature (ctrl_t ctrl,
   /* Get rid of old OCSP specific certificate references. */
   release_ctrl_ocsp_certs (ctrl);
 
-  if (signer_fpr)
+  if (signer_fpr_list && !signer_fpr_list->next)
     {
-      /* We use the default OCSP responder's certificate.  Get it from
-         the fingerprint. */
-      cert = get_cert_byhexfpr (signer_fpr);
+      /* There is exactly one signer fingerprint given. Thus we use
+         the default OCSP responder's certificate and instantly know
+         the certificate to use.  */
+      cert = get_cert_byhexfpr (signer_fpr_list->hexfpr);
       if (!cert)
-        cert = get_cert_local (ctrl, signer_fpr);
+        cert = get_cert_local (ctrl, signer_fpr_list->hexfpr);
       if (cert)
         {
-          err = check_signature_core (ctrl, cert, s_sig, s_hash, signer_fpr);
+          err = check_signature_core (ctrl, cert, s_sig, s_hash,
+                                      signer_fpr_list);
           ksba_cert_release (cert);
           cert = NULL;
           if (!err)
@@ -463,7 +470,7 @@ check_signature (ctrl_t ctrl,
             xfree (cref);
         }
 
-      /* Get the certificate by means of the respinder ID. */
+      /* Get the certificate by means of the responder ID. */
       err = ksba_ocsp_get_responder_id (ocsp, &name, &keyid);
       if (err)
         {
@@ -490,7 +497,8 @@ check_signature (ctrl_t ctrl,
 
       if (cert)
         {
-          err = check_signature_core (ctrl, cert, s_sig, s_hash, NULL);
+          err = check_signature_core (ctrl, cert, s_sig, s_hash,
+                                      signer_fpr_list);
           ksba_cert_release (cert);
           if (!err)
             {
@@ -508,9 +516,11 @@ check_signature (ctrl_t ctrl,
 
 /* Check whether the certificate either given by fingerprint CERT_FPR
    or directly through the CERT object is valid by running an OCSP
-   transaction.  */
+   transaction.  With FORCE_DEFAULT_RESPONDER set only the configured
+   default responder is used. */
 gpg_error_t
-ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr)
+ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr,
+              int force_default_responder)
 {
   gpg_error_t err;
   ksba_ocsp_t ocsp = NULL;
@@ -528,7 +538,7 @@ ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr)
   int i, idx;
   char *oid;
   ksba_name_t name;
-  const char *default_signer = NULL;
+  fingerprint_list_t default_signer = NULL;
 
   /* Get the certificate.  */
   if (cert)
@@ -578,7 +588,7 @@ ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr)
      2. If this fails use the default responder, if any.
    */
   url = NULL;
-  for (idx=0; !url && !opt.ignore_ocsp_service_url
+  for (idx=0; !url && !opt.ignore_ocsp_service_url && !force_default_responder
          && !(err=ksba_cert_get_authority_info_access (cert, idx,
                                                        &oid, &name)); idx++)
     {
@@ -610,7 +620,7 @@ ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr)
           err = gpg_error (GPG_ERR_CONFIGURATION);
           goto leave;
         }
-      if (!opt.ocsp_signer || !*opt.ocsp_signer)
+      if (!opt.ocsp_signer)
         {
           log_info (_("no default OCSP signer defined\n"));
           err = gpg_error (GPG_ERR_CONFIGURATION);
@@ -717,31 +727,51 @@ ocsp_isvalid (ctrl_t ctrl, ksba_cert_t cert, const char *cert_fpr)
 
   if (status == KSBA_STATUS_REVOKED)
     err = gpg_error (GPG_ERR_CERT_REVOKED);
+  else if (status == KSBA_STATUS_UNKNOWN)
+    err = gpg_error (GPG_ERR_NO_DATA);
   else if (status != KSBA_STATUS_GOOD)
     err = gpg_error (GPG_ERR_GENERAL);
 
+  /* Allow for some clock skew. */
   get_isotime (current_time);
-  /* Allow for some clock skew. Note, that NEXT_UPDATE is
-     optional. */
   add_isotime (current_time, opt.ocsp_max_clock_skew);
-  if (*next_update && strcmp (next_update, current_time) < 0 )
+
+  if (strcmp (this_update, current_time) > 0 )
     {
-      log_error (_("OCSP responder returned an too old status\n"));
-      log_info ("now: %s  next_update: %s\n", current_time, next_update);
+      log_error (_("OCSP responder returned a status in the future\n"));
+      log_info ("used now: %s  this_update: %s\n", current_time, this_update);
       if (!err)
         err = gpg_error (GPG_ERR_TIME_CONFLICT);
     }
+
   /* Check that THIS_UPDATE is not too far back in the past. */
   copy_time (tmp_time, this_update);
-  add_isotime (this_update, opt.ocsp_current_period + opt.ocsp_max_clock_skew);
-  if (!*this_update || strcmp (this_update, current_time) < 0 )
+  add_isotime (tmp_time, opt.ocsp_max_period+opt.ocsp_max_clock_skew);
+  if (!*tmp_time || strcmp (tmp_time, current_time) < 0 )
     {
       log_error (_("OCSP responder returned a non-current status\n"));
-      log_info ("now: %s  this_update: %s\n", current_time, tmp_time);
+      log_info ("used now: %s  this_update: %s\n", 
+                current_time, this_update);
       if (!err)
         err = gpg_error (GPG_ERR_TIME_CONFLICT);
     }
 
+  /* Check that we are not beyound NEXT_UPDATE  (plus some extra time). */
+  if (*next_update)
+    {
+      copy_time (tmp_time, next_update);
+      add_isotime (tmp_time, opt.ocsp_current_period+opt.ocsp_max_clock_skew);
+      if (!*tmp_time && strcmp (tmp_time, current_time) < 0 )
+        {
+          log_error (_("OCSP responder returned an too old status\n"));
+          log_info ("used now: %s  next_update: %s\n", 
+                    current_time, next_update);
+          if (!err)
+            err = gpg_error (GPG_ERR_TIME_CONFLICT);
+        }
+    }
+
+
  leave:
   gcry_md_close (md);
   gcry_sexp_release (s_sig);
